import { CarBlockIterator, CarWriter } from '@ipld/car';
import * as dagCBOR from '@ipld/dag-cbor';
import varint from 'varint';
/**
 * @typedef {import('@ipld/unixfs').Block} Block
 */
export const code = 0x0202;
/** Byte length of a CBOR encoded CAR header with zero roots. */
const NO_ROOTS_HEADER_LENGTH = 18;
/** @param {import('./types.js').AnyLink} [root] */
export function headerEncodingLength(root) {
    if (!root)
        return NO_ROOTS_HEADER_LENGTH;
    const headerLength = dagCBOR.encode({ version: 1, roots: [root] }).length;
    const varintLength = varint.encodingLength(headerLength);
    return varintLength + headerLength;
}
/** @param {Block} block */
export function blockHeaderEncodingLength(block) {
    const payloadLength = block.cid.bytes.length + block.bytes.length;
    const varintLength = varint.encodingLength(payloadLength);
    return varintLength + block.cid.bytes.length;
}
/** @param {Block} block */
export function blockEncodingLength(block) {
    return blockHeaderEncodingLength(block) + block.bytes.length;
}
/**
 * @param {Iterable<Block> | AsyncIterable<Block>} blocks
 * @param {import('./types.js').AnyLink} [root]
 * @returns {Promise<import('./types.js').CARFile>}
 */
export async function encode(blocks, root) {
    // @ts-expect-error
    const { writer, out } = CarWriter.create(root);
    /** @type {Error?} */
    let error;
    void (async () => {
        try {
            for await (const block of blocks) {
                await writer.put(block);
            }
        }
        catch ( /** @type {any} */err) {
            error = err;
        }
        finally {
            await writer.close();
        }
    })();
    const chunks = [];
    for await (const chunk of out)
        chunks.push(chunk);
    // @ts-expect-error
    if (error != null)
        throw error;
    const roots = root != null ? [root] : [];
    return Object.assign(new Blob(chunks), { version: 1, roots });
}
/** @extends {ReadableStream<Block>} */
export class BlockStream extends ReadableStream {
    /** @param {import('./types.js').BlobLike} car */
    constructor(car) {
        /** @type {Promise<CarBlockIterator>?} */
        let blocksPromise = null;
        const getBlocksIterable = () => {
            if (blocksPromise)
                return blocksPromise;
            blocksPromise = CarBlockIterator.fromIterable(toIterable(car.stream()));
            return blocksPromise;
        };
        /** @type {AsyncIterator<Block>?} */
        let iterator = null;
        super({
            async start() {
                const blocks = await getBlocksIterable();
                iterator = /** @type {AsyncIterator<Block>} */ (blocks[Symbol.asyncIterator]());
            },
            async pull(controller) {
                /* c8 ignore next */
                if (!iterator)
                    throw new Error('missing blocks iterator');
                const { value, done } = await iterator.next();
                if (done)
                    return controller.close();
                controller.enqueue(value);
            },
        });
        /** @returns {Promise<import('./types.js').AnyLink[]>} */
        this.getRoots = async () => {
            const blocks = await getBlocksIterable();
            return await blocks.getRoots();
        };
    }
}
/* c8 ignore start */
/**
 * {@link ReadableStream} is an async iterable in newer environments, but it's
 * not standard yet. This function normalizes a {@link ReadableStream} to a
 * definite async iterable.
 *
 * @template T
 * @param {ReadableStream<T> | AsyncIterable<T>} stream
 * @returns {AsyncIterable<T>} An async iterable of the contents of the
 *                             {@link stream} (possibly {@link stream} itself).
 */
function toIterable(stream) {
    return Symbol.asyncIterator in stream
        ? stream
        : (async function* () {
            const reader = stream.getReader();
            try {
                while (true) {
                    const { done, value } = await reader.read();
                    if (done)
                        return;
                    yield value;
                }
            }
            finally {
                reader.releaseLock();
            }
        })();
}
/* c8 ignore end */
//# sourceMappingURL=car.js.map